﻿
using UnityEngine;
using System.Collections;


public class PlayerControl : MonoBehaviour
{
    // movement config
    public float pushForce = 20f;
    public float gravity = -15f;
    public float runSpeed = 8f;
    public float groundDamping = 20f; // how fast do we change direction? higher means faster
    public float inAirDamping = 5f;
    public float targetJumpHeight = 2f;
    public float lastDashStart = float.NegativeInfinity;
    [HideInInspector]
    public float currentDashDirection = 0.0f;
    [HideInInspector]
    public float dashDirection = 1.0f;
    public float dashCompletionTime = 0.25f;
    public float dashStartSpeed = 20.0f;
    public float dashEndSpeed = 8.0f;
    public float dashCoolDown = 0.5f;
    // public float jumpWaitTime = 2.0;

    [HideInInspector]
    public float rawMovementDirection = 1;
    //[HideInInspector]
    public float normalizedHorizontalSpeed = 0;

    CharacterController2D _controller;
    public RaycastHit2D lastControllerColliderHit;

    [HideInInspector]
    public Vector3 velocity;

    public int playerNumber = 1; //gibt an, ob es sich um player one, player two, etc. handelt. sollte nicht 0 sein;
    KeyCode goRightKeyCode, goLeftKeyCode, jumpKeyCode, dashKeyCode, shootKeyCode;
    private Rigidbody2D rigidbody;

    void Start()
    {
        rigidbody = GetComponent<Rigidbody2D>();
        _controller = GetComponent<CharacterController2D>();
        _controller.onControllerCollidedEvent += onControllerCollider;

        switch (playerNumber)
        {
            case 1:

                setKeyCodes(KeyCode.D, KeyCode.A, KeyCode.W, KeyCode.S, KeyCode.LeftShift);
                break;
            case 2:

                setKeyCodes(KeyCode.RightArrow, KeyCode.LeftArrow, KeyCode.UpArrow, KeyCode.DownArrow, KeyCode.RightShift);
                break;
            //TODO player 3 and 4
            default:
                break;
        }
    }


    private void setKeyCodes(KeyCode right, KeyCode left, KeyCode up, KeyCode dash, KeyCode shoot)
    {
        goRightKeyCode = right;
        goLeftKeyCode = left;
        jumpKeyCode = up;


        dashKeyCode = dash;
        shootKeyCode = shoot;
    }

    void onControllerCollider(RaycastHit2D hit)
    {
        Rigidbody2D body = hit.collider.gameObject.GetComponent<Rigidbody2D>();
        if (body)
        {
            //TODO abfrage " is dashing"
            body.AddForce(_controller.getDirectionOfMovement() * pushForce);
            //TODO set is dashing to false
        }
    }

    void Update()
    {
        // grab our current velocity to use as a base for all calculations
        velocity = _controller.velocity;

        if (_controller.isGrounded)
            velocity.y = 0;

        if (Input.GetKey (goRightKeyCode))
        {
          dashDirection = 1.0f;
        }
        else if (Input.GetKey (goLeftKeyCode))
        {
          dashDirection = -1.0f;
        }

        if (canDash () && Input.GetKey (dashKeyCode))
        {
          lastDashStart = Time.time;
          currentDashDirection = dashDirection;
        }

        if (isDashing ())
        {
            velocity.x = Mathf.Lerp(currentDashDirection * dashStartSpeed, currentDashDirection * dashEndSpeed, getDashTime() / dashCompletionTime);
            velocity.y = 0;
        }
        else
        {
          if (Input.GetKey(goRightKeyCode))
          {
              normalizedHorizontalSpeed = 1;
              if (transform.localScale.x < 0f)
                  transform.localScale = new Vector3(-transform.localScale.x, transform.localScale.y, transform.localScale.z);

              //if (_controller.isGrounded)
          }
          else if (Input.GetKey(goLeftKeyCode))
          {
              normalizedHorizontalSpeed = -1;
              if (transform.localScale.x > 0f)
                  transform.localScale = new Vector3(-transform.localScale.x, transform.localScale.y, transform.localScale.z);

              //if (_controller.isGrounded)
          }
          else
          {
              normalizedHorizontalSpeed = 0;

              //if (_controller.isGrounded)
          }

          if (Input.GetKey (jumpKeyCode))
          {
              //to avoid DOUBLE JUMP
              if (_controller.isGrounded)
              {
                  velocity.y = Mathf.Sqrt(2f * targetJumpHeight * -gravity);
              }
          }


          // apply horizontal speed smoothing it
          var smoothedMovementFactor = _controller.isGrounded ? groundDamping : inAirDamping; // how fast do we change direction?
          velocity.x = Mathf.Lerp(velocity.x, normalizedHorizontalSpeed * rawMovementDirection * runSpeed, Time.deltaTime * smoothedMovementFactor);

          // apply gravity before moving
          velocity.y += gravity * Time.deltaTime;

          /*==========Power Ups // Bullet management ===*/
          if (Input.GetKeyDown(shootKeyCode))
          {
            Debug.Log("Shoot pressed. Can shoot: " + canShoot());
            if (canShoot())
            {
              shoot();
              coolDown();
            }
          }
        }

        _controller.move(velocity * Time.deltaTime);
    }

    private float getDashTime ()
    {
        return Time.time - lastDashStart;
    }

    private bool isDashing ()
    {
        float dashTime = getDashTime();
        return 0 <= dashTime && dashTime < dashCompletionTime;
    }

    private bool canDash ()
    {
        return getDashTime() >= dashCompletionTime + dashCoolDown;
    }

    /*==========Power Ups // Bullet management ===*/

    public const int POWERUP_NONE = 0;
    public const int POWERUP_HEALINGBULLET = 1;
    public const int POWERUP_TRAPDESTROYER = 2;

    private int powerUpType = POWERUP_NONE;

    private bool coolingDown = false;

    public float coolDownTime = 2.0f;

    public GameObject healingBulletPrefab;
    public GameObject trapDestroyingBulletPrefab;

    public float spawnDistance = 1.0f;
    public void setPowerUpType(int type)
    {
        powerUpType = type;

    }

    private void shoot()
    {
        float dir = Mathf.Sign(transform.localScale.x);
        Vector2 pos = new Vector2(transform.position.x + dir * spawnDistance, transform.position.y);

        switch (powerUpType)
        {
            case POWERUP_HEALINGBULLET:
                GameObject bullet = Instantiate(healingBulletPrefab, pos, transform.rotation) as GameObject;
                bullet.GetComponent<Bullet>().shoot(new Vector2(dir, 0));
                break;
            case POWERUP_TRAPDESTROYER:
                GameObject trapbullet = Instantiate(trapDestroyingBulletPrefab, pos, transform.rotation) as GameObject;
                trapbullet.GetComponent<Bullet>().shoot(new Vector2(dir, 0));
                break;
            default:
                break;
        }
    }

    private void coolDown()
    {
        coolingDown = true;
        Invoke("coolDownOver", coolDownTime);
    }

    private void coolDownOver()
    {
        coolingDown = false;
    }

    private bool canShoot()
    {
        return (!coolingDown) && powerUpType != POWERUP_NONE;
    }

}
